package de.ozgcloud.admin.migration;

import io.mongock.api.annotations.ChangeUnit;
import io.mongock.api.annotations.Execution;
import io.mongock.api.annotations.RollbackExecution;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;

@ChangeUnit(id = "2024-02-20 17:00:00 OZG-4948-OZG-5058", order = "M001", author = "lmonnerjahn", runAlways = true)
public class M001_CreateEmptyPostfachIfMissing {  // NOSONAR
	static final String SETTINGS_COLLECTION = "settings";

	static final String TYPE_NAME_KEY = "name";
	static final String TYPE_NAME_POSTFACH_VALUE = "Postfach";

	@Execution
	public void doMigration(MongoTemplate template) {
		template.upsert(buildQuery(), buildUpdate(), SETTINGS_COLLECTION);
	}

	private Query buildQuery() {
		var criteria = Criteria.where(TYPE_NAME_KEY).is(TYPE_NAME_POSTFACH_VALUE);
		return Query.query(criteria);
	}

	private Update buildUpdate() {
		var update = new Update();
		update.setOnInsert(TYPE_NAME_KEY, TYPE_NAME_POSTFACH_VALUE);
		return update;
	}

	@RollbackExecution
	public void rollback() {
		// kein rollback implementiert
	}
}
