/*
 * Copyright (c) 2024. Das Land Schleswig-Holstein vertreten durch das Ministerium für Energiewende, Klimaschutz, Umwelt und Natur
 * Zentrales IT-Management
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.nativeimage;

import static org.springframework.aot.hint.MemberCategory.*;

import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.aot.generate.GenerationContext;
import org.springframework.aot.hint.ReflectionHints;
import org.springframework.beans.factory.aot.BeanFactoryInitializationAotContribution;
import org.springframework.beans.factory.aot.BeanFactoryInitializationAotProcessor;
import org.springframework.beans.factory.aot.BeanFactoryInitializationCode;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.boot.autoconfigure.AutoConfigurationPackages;
import org.springframework.context.annotation.ClassPathScanningCandidateComponentProvider;
import org.springframework.core.type.filter.AnnotationTypeFilter;
import org.springframework.util.ClassUtils;

import io.mongock.api.annotations.ChangeUnit;
import io.mongock.runner.springboot.MongockSpringboot;
import io.mongock.utils.StringUtils;

public class MigrationUnitAotProcessor implements BeanFactoryInitializationAotProcessor {
	@Override
	public BeanFactoryInitializationAotContribution processAheadOfTime(ConfigurableListableBeanFactory beanFactory) {
		if (AutoConfigurationPackages.has(beanFactory)) {
			var scanner = createScanner();
			Set<Class<?>> unitSet = AutoConfigurationPackages.get(beanFactory)
					.stream()
					.filter(StringUtils::hasText)
					.flatMap(packageToScan -> scanner.findCandidateComponents(packageToScan).stream())
					.map(beanDefinition -> {
						try {
							return ClassUtils.forName(Objects.requireNonNull(beanDefinition.getBeanClassName()),
									beanFactory.getBeanClassLoader());
						} catch (ClassNotFoundException e) {
							throw new RuntimeException(e);
						}
					}).collect(Collectors.toSet());
			return (GenerationContext generationContext, BeanFactoryInitializationCode beanFactoryInitializationCode) ->
					registerReflectiveHints(generationContext.getRuntimeHints().reflection(), unitSet);
		}
		return null;
	}

	private ClassPathScanningCandidateComponentProvider createScanner() {
		var scanner = new ClassPathScanningCandidateComponentProvider(false);
		scanner.addIncludeFilter(new AnnotationTypeFilter(ChangeUnit.class));
		return scanner;
	}

	private void registerReflectiveHints(ReflectionHints hints, Set<Class<?>> migrationUnits) {
		for (var unit : migrationUnits) {
			hints.registerType(unit, hint ->
					hint.withMembers(INVOKE_DECLARED_METHODS, INVOKE_DECLARED_CONSTRUCTORS)
			);
		}
		hints.registerType(MongockSpringboot.RunnerSpringbootBuilderImpl.class, hint ->
				hint.withMembers(INVOKE_DECLARED_METHODS, INVOKE_DECLARED_CONSTRUCTORS)
		);
	}
}
